#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gtk/gtk.h>
#include <libgnome/gnome-desktop-item.h>
#include <libgnomeui/libgnomeui.h>
#include <panel-applet.h>
#include <string.h>

#include "icon-loader.h"
#include "main-menu-button.h"
#include "panel-stock-icons.h"
#include "window-list.h"
#include "window-menu-button.h"

BeIconLoader *icon_loader = NULL;

void init(void) {
	static gboolean has_run = FALSE;

	if(!has_run) {
		has_run = TRUE;

		panel_init_stock_icons_and_items();

		icon_loader = be_icon_loader_new(NULL);
		g_object_ref(G_OBJECT(icon_loader));
	}
}

static void about_dialog_destroyed(GtkObject *about_dialog, GtkWidget *widget) {
	g_object_set_data(G_OBJECT(widget), "AppletVerbs:AboutDialog", NULL);
}

static void display_about_dialog(BonoboUIComponent *uic, GtkWidget *widget, const gchar *verbname) {
	GtkWidget *about_dialog = g_object_get_data(G_OBJECT(widget), "AppletVerbs:AboutDialog");

	if(about_dialog) {
		gtk_window_set_screen(GTK_WINDOW(about_dialog), gtk_widget_get_screen(widget));
		gtk_widget_show(about_dialog);
		gtk_window_present(GTK_WINDOW(about_dialog));
	} else {
		static const gchar *authors[] = {
			"Paul Varga <null_and_void@users.sourceforge.net>",
			NULL
		};
		const gchar **documenters = NULL;
		const gchar *translator_credits = NULL;
		GdkPixbuf *pixbuf = NULL;
		gchar *file;

		file = gnome_desktop_item_find_icon(NULL, "beos_logo", 48, 0);

		if(file) {
			pixbuf = gdk_pixbuf_new_from_file (file, NULL);
			g_free(file);
		}

		about_dialog = gnome_about_new("Be-Applet", VERSION,
			"Copyright \xc2\xa9 2004, Paul Varga.",
			_("A set of BeOS Deskbar emulation applets."),
			authors,
			documenters,
			translator_credits,
			pixbuf);

		gtk_window_set_screen(GTK_WINDOW(about_dialog), gtk_widget_get_screen(widget));

		if(pixbuf) {
			gtk_window_set_icon(GTK_WINDOW(about_dialog), pixbuf);
			g_object_unref(pixbuf);
		}

		g_object_set_data(G_OBJECT(widget), "AppletVerbs:AboutDialog", about_dialog);

		g_signal_connect(G_OBJECT(about_dialog), "destroy", G_CALLBACK(&about_dialog_destroyed), widget);

		gtk_widget_show(about_dialog);
	}
}

static gboolean be_applet_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data) {
	switch(event->button) {
		case 3:
			return TRUE;
		default:
			break;
	}

	return FALSE;
}

static const BonoboUIVerb button_applet_verbs[] = {
	BONOBO_UI_UNSAFE_VERB("MenuButtonAbout", &display_about_dialog),
/*	BONOBO_UI_UNSAFE_VERB("MenuButtonHelp", NULL), */
	BONOBO_UI_VERB_END
};

gboolean be_button_applet_fill(PanelApplet *applet) {
	GtkWidget *menu_root   = gtk_hbox_new(TRUE, 0);
	GtkWidget *button_menu = be_main_menu_button_new(icon_loader);
	GtkWidget *button_apps = be_window_menu_button_new();
	GtkWidget *widget;

	panel_applet_set_flags(applet, PANEL_APPLET_EXPAND_MINOR);

	panel_applet_setup_menu_from_file(
		applet,
		/* datadir */ NULL,
		"GNOME_BeMenuButtonApplet.xml",
		/* app_name */ NULL,
		button_applet_verbs,
		menu_root);

	widget = gtk_image_new_from_stock(PANEL_STOCK_GNOME_LOGO, panel_menu_bar_icon_get_size());
	g_signal_connect(widget, "button-press-event", G_CALLBACK(&be_applet_button_press_event), NULL);
	gtk_container_add(GTK_CONTAINER(button_menu), widget);

	widget = gtk_image_new_from_stock(PANEL_STOCK_WINDOW_MENU, panel_menu_bar_icon_get_size());
	g_signal_connect(widget, "button-press-event", G_CALLBACK(&be_applet_button_press_event), NULL);
	gtk_container_add(GTK_CONTAINER(button_apps), widget);

	gtk_container_add(GTK_CONTAINER(menu_root), button_menu);
	gtk_container_add(GTK_CONTAINER(menu_root), button_apps);

	gtk_container_add(GTK_CONTAINER(applet), menu_root);
	gtk_widget_show_all(GTK_WIDGET(applet));

	return TRUE;
}

static const BonoboUIVerb windows_applet_verbs[] = {
	BONOBO_UI_UNSAFE_VERB("WindowListAbout", &display_about_dialog),
/*	BONOBO_UI_UNSAFE_VERB("WindowListHelp", NULL), */
	BONOBO_UI_VERB_END
};

gboolean be_windows_applet_fill(PanelApplet *applet) {
	GtkWidget *window_list = be_window_list_new();

	panel_applet_set_flags(applet, PANEL_APPLET_EXPAND_MINOR | PANEL_APPLET_HAS_HANDLE);

	panel_applet_setup_menu_from_file(
		applet,
		/* datadir */ NULL,
		"GNOME_BeWindowListApplet.xml",
		/* app_name */ NULL,
		windows_applet_verbs,
		window_list);

	gtk_container_add(GTK_CONTAINER(applet), window_list);
	gtk_widget_show_all(GTK_WIDGET(applet));

	return TRUE;
}

gboolean be_applet_factory(PanelApplet *applet, const char *iid, gpointer data) {
	init();

	if(strcmp(iid, "OAFIID:Be_Button_Applet") == 0)
		return be_button_applet_fill(applet);
	else if(strcmp(iid, "OAFIID:Be_Windows_Applet") == 0)
		return be_windows_applet_fill(applet);
	
	return FALSE;
}

PANEL_APPLET_BONOBO_FACTORY("OAFIID:Be_Applet_Factory",
                            PANEL_TYPE_APPLET,
                            "BeOS Emulation Applets",
                            "0",
                            &be_applet_factory,
                            NULL);
