#include "config.h"

#include "conf.h"

#define DESKTOP_GNOME_LOCKDOWN_DIR "/desktop/gnome/lockdown"

GConfClient *be_conf_get_client(void) {
	static GConfClient *client = NULL;

	if(!client) {
		client = gconf_client_get_default();

		/* lockdown */
		gconf_client_add_dir(
			be_conf_get_client(),
			DESKTOP_GNOME_LOCKDOWN_DIR,
			GCONF_CLIENT_PRELOAD_ONELEVEL,
			NULL);
	}

	return client;
}

/* panel stuff */
#define PANEL_CONFIG_DIR           "/apps/panel/profiles"
#define PANEL_SCHEMAS_DIR          "/schemas/apps/panel"
#define PANEL_DEFAULTS_DIR         "/apps/panel/default_setup"
#define PANEL_GLOBAL_DIR           "/apps/panel/global"

static gchar *panel_profile = NULL;

void be_conf_panel_profile_set(const gchar *profile) {
	if(!profile)
		profile = "default";

	if(panel_profile) {
		gchar *dir = gconf_concat_dir_and_key(PANEL_CONFIG_DIR, panel_profile);
		gchar *key = g_strdup_printf("%s/general", dir);

		gconf_client_remove_dir(
			be_conf_get_client(),
			key,
			NULL);

		g_free(dir);
		g_free(key);
		g_free(panel_profile);
	}

	panel_profile = g_strdup(profile);

	if(panel_profile) {
		gchar *dir = gconf_concat_dir_and_key(PANEL_CONFIG_DIR, panel_profile);
		gchar *key = g_strdup_printf("%s/general", dir);

		gconf_client_add_dir(
			be_conf_get_client(),
			key,
			GCONF_CLIENT_PRELOAD_ONELEVEL,
			NULL);

		g_free(dir);
		g_free(key);
	}
}
G_CONST_RETURN gchar *be_conf_panel_profile_get(void) {
	if(!panel_profile)
		be_conf_panel_profile_set(NULL);

	return panel_profile;
}

static gchar *be_conf_panel_get_global_key(const gchar *key) {
	return g_strdup_printf("/apps/panel/global/%s", key);
}
static gchar *be_conf_panel_get_general_key(const gchar *key) {
	return g_strdup_printf(PANEL_CONFIG_DIR "/%s/general/%s",
		be_conf_panel_profile_get(),
		key);
}

static gboolean be_conf_panel_profile_get_program_list_setting(const gchar *setting) {
	gchar    *key;
	gboolean  value;

	key = be_conf_panel_get_general_key(setting);

	value = gconf_client_get_bool(
		be_conf_get_client(),
		key,
		NULL);

	g_free(key);

	return value;
}
gboolean be_conf_panel_profile_get_show_program_list(void) {
	return be_conf_panel_profile_get_program_list_setting("show_program_list");
}
gboolean be_conf_panel_profile_get_enable_program_list(void) {
	return be_conf_panel_profile_get_program_list_setting("enable_program_list");
}

gboolean be_conf_panel_profile_can_set_show_program_list(void) {
	gchar    *key;
	gboolean  can_set;

	key = be_conf_panel_get_general_key("show_program_list");

	can_set = gconf_client_key_is_writable(
		be_conf_get_client(),
		key,
		NULL);

	g_free(key);

	return can_set;
}

void be_conf_panel_profile_set_show_program_list(gboolean show_program_list) {
	gchar    *key;
	gboolean  can_set;

	key = be_conf_panel_get_general_key("show_program_list");

	can_set = gconf_client_set_bool(
		be_conf_get_client(),
		key,
		show_program_list,
		NULL);

	g_free(key);
}

/* lockdown stuff */
#define DISABLE_COMMAND_LINE_KEY   DESKTOP_GNOME_LOCKDOWN_DIR "/disable_command_line"

gboolean be_conf_panel_lockdown_get_disable_command_line(void) {
	return gconf_client_get_bool(
		be_conf_get_client(),
		DISABLE_COMMAND_LINE_KEY,
		NULL);
}
