#include "config.h"

#include "custom-file-menu-item.h"

static void be_custom_file_menu_item_instance_init(GTypeInstance *instance, gpointer g_class) {
	BeCustomFileMenuItem *self = (BeCustomFileMenuItem *) instance;

	/* Do stuff */
	self->custom_text        = NULL;
	self->custom_image_stock = NULL;
}

static GObject *be_custom_file_menu_item_constructor(GType type, guint n_construct_properties, GObjectConstructParam *construct_properties) {
	GObject *obj;

	{ /* Invoke parent constructor. */
		BeCustomFileMenuItemClass *klass        = BE_CUSTOM_FILE_MENU_ITEM_CLASS(g_type_class_peek(BE_CUSTOM_FILE_MENU_ITEM_TYPE));
		GObjectClass   *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

		obj = parent_class->constructor(type, n_construct_properties, construct_properties);
	}

	/* Do stuff */

	return obj;
}

static void be_custom_file_menu_item_dispose(GObject *object) {
/*	BeCustomFileMenuItem      *self         = BE_CUSTOM_FILE_MENU_ITEM(object); */
	BeCustomFileMenuItemClass *klass        = BE_CUSTOM_FILE_MENU_ITEM_CLASS(g_type_class_peek(BE_CUSTOM_FILE_MENU_ITEM_TYPE));
	GObjectClass              *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */

	/* Invoke parent dispose */
	parent_class->dispose(object);
}

static void be_custom_file_menu_item_finalize(GObject *object) {
	BeCustomFileMenuItem      *self         = BE_CUSTOM_FILE_MENU_ITEM(object);
	BeCustomFileMenuItemClass *klass        = BE_CUSTOM_FILE_MENU_ITEM_CLASS(g_type_class_peek(BE_CUSTOM_FILE_MENU_ITEM_TYPE));
	GObjectClass              *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */
	g_free(self->custom_text);
	g_free(self->custom_image_stock);

	/* Invoke parent finalize */
	parent_class->finalize(object);
}

static void be_custom_file_menu_item_set_text_virtual(BeFileMenuItem *file_menu_item) {
	BeCustomFileMenuItem      *self         = BE_CUSTOM_FILE_MENU_ITEM(file_menu_item);
	BeCustomFileMenuItemClass *klass        = BE_CUSTOM_FILE_MENU_ITEM_CLASS(g_type_class_peek(BE_CUSTOM_FILE_MENU_ITEM_TYPE));
	BeFileMenuItemClass       *parent_class = BE_FILE_MENU_ITEM_CLASS(g_type_class_peek_parent(klass));

	if(!self->custom_text) {
		/* Invoke parent set_text */
		parent_class->set_text(file_menu_item);
	}
}
static void be_custom_file_menu_item_set_image_virtual(BeFileMenuItem *file_menu_item) {
	BeCustomFileMenuItem      *self         = BE_CUSTOM_FILE_MENU_ITEM(file_menu_item);
	BeCustomFileMenuItemClass *klass        = BE_CUSTOM_FILE_MENU_ITEM_CLASS(g_type_class_peek(BE_CUSTOM_FILE_MENU_ITEM_TYPE));
	BeFileMenuItemClass       *parent_class = BE_FILE_MENU_ITEM_CLASS(g_type_class_peek_parent(klass));

	if(self->custom_image_stock) {
		GtkWidget   *image     = gtk_image_menu_item_get_image(GTK_IMAGE_MENU_ITEM(file_menu_item));
		gchar       *stock_id;
		GtkIconSize  icon_size;

		gtk_image_get_stock(GTK_IMAGE(image), &stock_id, &icon_size);

		if(file_menu_item->icon_size != icon_size) {
			image = gtk_image_new_from_stock(self->custom_image_stock, file_menu_item->icon_size);

			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(self), image);
		}
	} else {
		/* Invoke parent set_image */
		parent_class->set_image(file_menu_item);
	}
}

static void be_custom_file_menu_item_class_init(gpointer g_class, gpointer g_class_data) {
	GObjectClass        *gobject_class        = G_OBJECT_CLASS(g_class);
	BeFileMenuItemClass *file_menu_item_class = BE_FILE_MENU_ITEM_CLASS(g_class);

	gobject_class->constructor      = &be_custom_file_menu_item_constructor;
	gobject_class->dispose          = &be_custom_file_menu_item_dispose;
	gobject_class->finalize         = &be_custom_file_menu_item_finalize;

	file_menu_item_class->set_text  = &be_custom_file_menu_item_set_text_virtual;
	file_menu_item_class->set_image = &be_custom_file_menu_item_set_image_virtual;
}

GType be_custom_file_menu_item_get_type(void) {
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo info = {
			sizeof(BeCustomFileMenuItemClass),       /* class_size */
			NULL,                                    /* base_init */
			NULL,                                    /* base_finalize */
			&be_custom_file_menu_item_class_init,    /* class_init */
			NULL,                                    /* class_destroy */
			NULL,                                    /* class_data */
			sizeof(BeCustomFileMenuItem),            /* instance_size */
			0,                                       /* n_preallocs */
			&be_custom_file_menu_item_instance_init, /* instance_init */
			NULL                                     /* value_table */
		};

		type = g_type_register_static(BE_FILE_MENU_ITEM_TYPE, "BeCustomFileMenuItemType", &info, 0);
	}

	return type;
}

GtkWidget *be_custom_file_menu_item_new(const gchar *uri, BeIconLoader *icon_loader, GtkIconSize icon_size) {
	GtkWidget *custom_file_menu_item = gtk_widget_new(BE_CUSTOM_FILE_MENU_ITEM_TYPE,
		"uri", uri,
		"icon-loader", icon_loader,
		"icon-size", icon_size,
		NULL);

	return custom_file_menu_item;
}

void be_custom_file_menu_item_set_text(BeCustomFileMenuItem *custom_file_menu_item, const gchar *text) {
	g_free(custom_file_menu_item->custom_text);

	if(text) {
		GtkWidget *label = gtk_bin_get_child(GTK_BIN(custom_file_menu_item));

		custom_file_menu_item->custom_text = g_strdup(text);

		gtk_label_set_text(GTK_LABEL(label), custom_file_menu_item->custom_text);
	} else {
		custom_file_menu_item->custom_text = NULL;

		BE_FILE_MENU_ITEM_GET_CLASS(custom_file_menu_item)->set_text(BE_FILE_MENU_ITEM(custom_file_menu_item));
	}
}
void be_custom_file_menu_item_set_image_from_stock(BeCustomFileMenuItem *custom_file_menu_item, const gchar *stock_id) {
	BeFileMenuItem *file_menu_item = BE_FILE_MENU_ITEM(custom_file_menu_item);

	g_free(custom_file_menu_item->custom_image_stock);

	if(stock_id) {
		GtkWidget *image;

		custom_file_menu_item->custom_image_stock = g_strdup(stock_id);

		image = gtk_image_new_from_stock(custom_file_menu_item->custom_image_stock, file_menu_item->icon_size);

		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(custom_file_menu_item), image);
	} else {
		custom_file_menu_item->custom_image_stock = NULL;

		BE_FILE_MENU_ITEM_GET_CLASS(custom_file_menu_item)->set_image(file_menu_item);
	}
}
