#include "config.h"

#include "file-menu-item.h"

#include <libgnome/gnome-desktop-item.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include <string.h>

#include "dir-menu.h"

enum {
	BE_FILE_MENU_ITEM_PROPERTY_URI         = 1,
	BE_FILE_MENU_ITEM_PROPERTY_ICON_LOADER,
	BE_FILE_MENU_ITEM_PROPERTY_ICON_SIZE
};

static void be_file_menu_item_instance_init(GTypeInstance *instance, gpointer g_class) {
	BeFileMenuItem *self = (BeFileMenuItem *) instance;

	/* Do stuff */
	self->icon_loader = NULL;
	self->icon_size   = GTK_ICON_SIZE_MENU;
	self->uri         = NULL;
	self->name        = NULL;
	self->icon        = NULL;
	self->action_type = BE_FILE_MENU_ITEM_ACTION_NONE;
	self->action_data = NULL;

	{ /* Add label */
		GtkWidget *accel_label = gtk_accel_label_new("");

		gtk_misc_set_alignment(GTK_MISC(accel_label), 0.0, 0.5);
		gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(accel_label), GTK_WIDGET(self));
		gtk_widget_show(accel_label);
		gtk_container_add(GTK_CONTAINER(self), accel_label);
	}
}

static void be_file_menu_item_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec) {
	BeFileMenuItem *self = BE_FILE_MENU_ITEM(object);

	switch(property_id) {
		case BE_FILE_MENU_ITEM_PROPERTY_URI:
			be_file_menu_item_set_uri(self, g_value_get_string(value));
			break;
		case BE_FILE_MENU_ITEM_PROPERTY_ICON_LOADER:
			be_file_menu_item_set_icon_loader(self, BE_ICON_LOADER(g_value_get_object(value)));
			break;
		case BE_FILE_MENU_ITEM_PROPERTY_ICON_SIZE:
			be_file_menu_item_set_icon_size(self, g_value_get_uint(value));
			break;
		default:
			g_assert_not_reached();
			break;
	}
}
static void be_file_menu_item_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec) {
	BeFileMenuItem *self = BE_FILE_MENU_ITEM(object);

	switch(property_id) {
		case BE_FILE_MENU_ITEM_PROPERTY_URI:
			g_value_set_string(value, self->uri);
			break;
		case BE_FILE_MENU_ITEM_PROPERTY_ICON_LOADER:
			g_value_set_object(value, self->icon_loader);
			break;
		case BE_FILE_MENU_ITEM_PROPERTY_ICON_SIZE:
			g_value_set_uint(value, self->icon_size);
			break;
		default:
			g_assert_not_reached();
			break;
	}
}

static void be_file_menu_item_activate(GtkMenuItem *menu_item) {
	BeFileMenuItem      *self         = BE_FILE_MENU_ITEM(menu_item);
	BeFileMenuItemClass *klass        = BE_FILE_MENU_ITEM_CLASS(g_type_class_peek(BE_FILE_MENU_ITEM_TYPE));
	GtkMenuItemClass    *parent_class = GTK_MENU_ITEM_CLASS(g_type_class_peek_parent(klass));

	if(!gtk_menu_item_get_submenu(menu_item)) {
		switch(self->action_type) {
			case BE_FILE_MENU_ITEM_ACTION_NONE:
				break;
			case BE_FILE_MENU_ITEM_ACTION_OPEN: {
				const gchar *data = self->action_data;

				if(!data)
					data = self->uri;

				if(!gnome_url_show(data, NULL))
					g_warning("Could not show URL: %s\n", data);
			}	break;
			case BE_FILE_MENU_ITEM_ACTION_SPAWN: {
				GdkScreen *screen  = gtk_widget_get_screen(GTK_WIDGET(menu_item));
				gchar     *data    = self->action_data;

				if(!gdk_spawn_command_line_on_screen(screen, data, NULL))
					g_warning("Could not spawn: %s\n", data);
			}	break;
		}
	}

	/* Invoke parent activate */
	if(parent_class->activate)
		parent_class->activate(menu_item);
}

static GObject *be_file_menu_item_constructor(GType type, guint n_construct_properties, GObjectConstructParam *construct_properties) {
	GObject *obj;

	{ /* Invoke parent constructor. */
		BeFileMenuItemClass *klass        = BE_FILE_MENU_ITEM_CLASS(g_type_class_peek(BE_FILE_MENU_ITEM_TYPE));
		GObjectClass   *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

		obj = parent_class->constructor(type, n_construct_properties, construct_properties);
	}

	/* Do stuff */

	return obj;
}

static void be_file_menu_item_dispose(GObject *object) {
	BeFileMenuItem      *self         = BE_FILE_MENU_ITEM(object);
	BeFileMenuItemClass *klass        = BE_FILE_MENU_ITEM_CLASS(g_type_class_peek(BE_FILE_MENU_ITEM_TYPE));
	GObjectClass        *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */
	if(self->icon_loader) {
		g_object_unref(self->icon_loader);
		self->icon_loader = NULL;
	}

	/* Invoke parent dispose */
	parent_class->dispose(object);
}

static void be_file_menu_item_finalize(GObject *object) {
	BeFileMenuItem      *self         = BE_FILE_MENU_ITEM(object);
	BeFileMenuItemClass *klass        = BE_FILE_MENU_ITEM_CLASS(g_type_class_peek(BE_FILE_MENU_ITEM_TYPE));
	GObjectClass        *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */
	g_free(self->uri);
	g_free(self->name);
	g_free(self->icon);
	g_free(self->action_data);

	/* Invoke parent finalize */
	parent_class->finalize(object);
}

static void be_file_menu_item_set_text(BeFileMenuItem *file_menu_item) {
	GtkWidget *label = gtk_bin_get_child(GTK_BIN(file_menu_item));

	if(file_menu_item->name)
		gtk_label_set_text(GTK_LABEL(label), file_menu_item->name);
	else
		gtk_label_set_text(GTK_LABEL(label), "");
}
static void be_file_menu_item_set_image(BeFileMenuItem *file_menu_item) {
	if(file_menu_item->icon_loader && file_menu_item->icon) {
		GtkWidget *image = be_icon_loader_image_new(file_menu_item->icon_loader, file_menu_item->icon, file_menu_item->icon_size);

		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(file_menu_item), image);
	}
}

static void be_file_menu_item_class_init(gpointer g_class, gpointer g_class_data) {
	GObjectClass        *gobject_class   = G_OBJECT_CLASS(g_class);
	GtkMenuItemClass    *menu_item_class = GTK_MENU_ITEM_CLASS(g_class);
	BeFileMenuItemClass *klass           = BE_FILE_MENU_ITEM_CLASS(g_class);
	GParamSpec          *pspec;

	gobject_class->constructor  = &be_file_menu_item_constructor;
	gobject_class->dispose      = &be_file_menu_item_dispose;
	gobject_class->finalize     = &be_file_menu_item_finalize;
	gobject_class->set_property = &be_file_menu_item_set_property;
	gobject_class->get_property = &be_file_menu_item_get_property;

	menu_item_class->activate   = &be_file_menu_item_activate;

	klass->set_text             = &be_file_menu_item_set_text;
	klass->set_image            = &be_file_menu_item_set_image;

	pspec = g_param_spec_string("uri",
		"URI",
		"URI of the file or directory the item is to represent",
		NULL,
		G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class,
		BE_FILE_MENU_ITEM_PROPERTY_URI,
		pspec);

	pspec = g_param_spec_object("icon-loader",
		"Icon loader",
		"Icon loader to use for requesting images",
		G_TYPE_OBJECT,
		G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class,
		BE_FILE_MENU_ITEM_PROPERTY_ICON_LOADER,
		pspec);

	pspec = g_param_spec_uint("icon-size",
		"Icon size",
		"Icon size to pass to the icon loader",
		0,
		~0U,
		GTK_ICON_SIZE_MENU,
		G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class,
		BE_FILE_MENU_ITEM_PROPERTY_ICON_SIZE,
		pspec);
}

static void be_file_menu_item_ready(BeReadyable *readyable) {
	BeFileMenuItem *file_menu_item = BE_FILE_MENU_ITEM(readyable);

	if(gtk_menu_item_get_submenu(GTK_MENU_ITEM(file_menu_item)))
		be_readyable_ready(BE_READYABLE(gtk_menu_item_get_submenu(GTK_MENU_ITEM(file_menu_item))));
}
static void be_file_menu_item_unready(BeReadyable *readyable) {
	BeFileMenuItem *file_menu_item = BE_FILE_MENU_ITEM(readyable);

	if(gtk_menu_item_get_submenu(GTK_MENU_ITEM(file_menu_item)))
		be_readyable_unready(BE_READYABLE(gtk_menu_item_get_submenu(GTK_MENU_ITEM(file_menu_item))));
}

static void be_file_menu_item_readyable_interface_init(gpointer g_iface, gpointer iface_data) {
	BeReadyableClass *klass = (BeReadyableClass *) g_iface;

	klass->ready   = &be_file_menu_item_ready;
	klass->unready = &be_file_menu_item_unready;
}

GType be_file_menu_item_get_type(void) {
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo info = {
			sizeof(BeFileMenuItemClass),      /* class_size */
			NULL,                             /* base_init */
			NULL,                             /* base_finalize */
			&be_file_menu_item_class_init,    /* class_init */
			NULL,                             /* class_destroy */
			NULL,                             /* class_data */
			sizeof(BeFileMenuItem),           /* instance_size */
			0,                                /* n_preallocs */
			&be_file_menu_item_instance_init, /* instance_init */
			NULL                              /* value_table */
		};
		static const GInterfaceInfo readyable_info = {
			(GInterfaceInitFunc) &be_file_menu_item_readyable_interface_init, /* interface_init */
			NULL,                                                             /* interface_finalize */
			NULL                                                              /* interface_data */
		};

		type = g_type_register_static(GTK_TYPE_IMAGE_MENU_ITEM, "BeFileMenuItemType", &info, 0);

		g_type_add_interface_static(type, BE_READYABLE_TYPE, &readyable_info);
	}

	return type;
}

GtkWidget *be_file_menu_item_new(const gchar *uri, BeIconLoader *icon_loader, GtkIconSize icon_size) {
	GtkWidget *file_menu_item = gtk_widget_new(BE_FILE_MENU_ITEM_TYPE,
		"uri", uri,
		"icon-loader", icon_loader,
		"icon-size", icon_size,
		NULL);

	return file_menu_item;
}

G_CONST_RETURN gchar *be_file_menu_item_get_name(const BeFileMenuItem *file_menu_item) {
	GtkWidget *label = gtk_bin_get_child(GTK_BIN(file_menu_item));
	return gtk_label_get_text(GTK_LABEL(label));
}

void be_file_menu_item_set_uri(BeFileMenuItem *file_menu_item, const gchar *uri) {
	GnomeVFSFileInfo *file_info;

	file_info = gnome_vfs_file_info_new();

	if(gnome_vfs_get_file_info(uri, file_info, GNOME_VFS_FILE_INFO_GET_MIME_TYPE | GNOME_VFS_FILE_INFO_FOLLOW_LINKS) != GNOME_VFS_OK) {
		g_warning("failed to get file info for \"%s\"", uri);
	} else {
		GnomeDesktopItem *desktop_item = NULL;
		gchar            *name = NULL;
		gchar            *icon = NULL;
		GtkWidget        *menu = NULL;

		g_free(file_menu_item->uri);
		file_menu_item->uri = g_strdup(uri);

		/* Cleanup */
		g_free(file_menu_item->name);
		g_free(file_menu_item->icon);
		file_menu_item->action_type = BE_FILE_MENU_ITEM_ACTION_NONE;

		g_free(file_menu_item->action_data);
		file_menu_item->action_data = NULL;

		if(gtk_menu_item_get_submenu(GTK_MENU_ITEM(file_menu_item)))
			gtk_menu_item_remove_submenu(GTK_MENU_ITEM(file_menu_item));

		/* Find new information */
		if(strcmp(gnome_vfs_file_info_get_mime_type(file_info), "application/x-gnome-app-info") == 0) {
			desktop_item = gnome_desktop_item_new_from_uri(uri, 0, NULL);
		} else if(strcmp(gnome_vfs_file_info_get_mime_type(file_info), "application/x-desktop") == 0) {
			desktop_item = gnome_desktop_item_new_from_uri(uri, 0, NULL);
		} else if(strcmp(gnome_vfs_file_info_get_mime_type(file_info), "x-directory/vfolder-desktop") == 0) {
			desktop_item = gnome_desktop_item_new_from_uri(uri, 0, NULL);

			menu = be_dir_menu_new(uri, file_menu_item->icon_loader, file_menu_item->icon_size);
		} else if(strcmp(gnome_vfs_file_info_get_mime_type(file_info), "x-directory/normal") == 0) {
			menu = be_dir_menu_new(uri, file_menu_item->icon_loader, file_menu_item->icon_size);
		} 

		if(desktop_item) {
			name = g_strdup(gnome_desktop_item_get_localestring(desktop_item, "Name"));
			icon = g_strdup(gnome_desktop_item_get_localestring(desktop_item, "Icon"));

			switch(gnome_desktop_item_get_entry_type(desktop_item)) {
				case GNOME_DESKTOP_ITEM_TYPE_APPLICATION: {
					const gchar *exec = gnome_desktop_item_get_string(desktop_item, "Exec");

					if(exec) {
						file_menu_item->action_type = BE_FILE_MENU_ITEM_ACTION_SPAWN;
						file_menu_item->action_data = g_strdup(exec);
					}
				}	break;
				case GNOME_DESKTOP_ITEM_TYPE_LINK: {
					const gchar *url = gnome_desktop_item_get_string(desktop_item, "URL");

					if(url) {
						file_menu_item->action_type = BE_FILE_MENU_ITEM_ACTION_OPEN;
						file_menu_item->action_data = g_strdup(url);
					}
				}	break;
				default:
					break;
			}

			gnome_desktop_item_unref(desktop_item);
		} else {
			GnomeIconLookupResultFlags result;

			name = g_strdup(file_info->name);
			icon = gnome_icon_lookup(gtk_icon_theme_get_default(), NULL, NULL, NULL, file_info,
				gnome_vfs_file_info_get_mime_type(file_info), GNOME_ICON_LOOKUP_FLAGS_NONE, &result);

			file_menu_item->action_type = BE_FILE_MENU_ITEM_ACTION_OPEN;
		}

		/* Set and cleanup new information */
		file_menu_item->name = name;
		file_menu_item->icon = icon;

		BE_FILE_MENU_ITEM_GET_CLASS(file_menu_item)->set_text(file_menu_item);
		BE_FILE_MENU_ITEM_GET_CLASS(file_menu_item)->set_image(file_menu_item);

		if(menu)
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_menu_item), menu);

		g_object_notify(G_OBJECT(file_menu_item), "uri");
	}

	gnome_vfs_file_info_unref(file_info);
}
G_CONST_RETURN gchar *be_file_menu_item_get_uri(const BeFileMenuItem *file_menu_item) {
	return file_menu_item->uri;
}

void be_file_menu_item_set_icon_loader(BeFileMenuItem *file_menu_item, BeIconLoader *icon_loader) {
	if(file_menu_item->icon_loader)
		g_object_unref(G_OBJECT(file_menu_item->icon_loader));

	file_menu_item->icon_loader = icon_loader;

	if(file_menu_item->icon_loader) {
		GtkWidget *submenu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(file_menu_item));

		g_object_ref(G_OBJECT(file_menu_item->icon_loader));

		BE_FILE_MENU_ITEM_GET_CLASS(file_menu_item)->set_image(file_menu_item);

		if(submenu) {
			be_dir_menu_set_icon_loader(BE_DIR_MENU(submenu), file_menu_item->icon_loader);

			if(GTK_WIDGET_VISIBLE(submenu))
				gtk_menu_reposition(GTK_MENU(submenu));
		}
	}

	g_object_notify(G_OBJECT(file_menu_item), "icon-loader");
}
BeIconLoader *be_file_menu_item_get_icon_loader(const BeFileMenuItem *file_menu_item) {
	return file_menu_item->icon_loader;
}

void be_file_menu_item_set_icon_size(BeFileMenuItem *file_menu_item, GtkIconSize icon_size) {
	if(file_menu_item->icon_size != icon_size) {
		GtkWidget *submenu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(file_menu_item));

		file_menu_item->icon_size = icon_size;

		BE_FILE_MENU_ITEM_GET_CLASS(file_menu_item)->set_image(file_menu_item);

		if(submenu) {
			be_dir_menu_set_icon_size(BE_DIR_MENU(submenu), file_menu_item->icon_size);

			if(GTK_WIDGET_VISIBLE(submenu))
				gtk_menu_reposition(GTK_MENU(submenu));
		}
	}
}
GtkIconSize be_file_menu_item_get_icon_size(const BeFileMenuItem *file_menu_item) {
	return file_menu_item->icon_size;
}
