#ifndef INC_BE_APPLET__FILE_MENU_ITEM_H
#define INC_BE_APPLET__FILE_MENU_ITEM_H

#include <gtk/gtk.h>
#include <libgnomevfs/gnome-vfs.h>

#include "icon-loader.h"
#include "readyable.h"

#define BE_FILE_MENU_ITEM_TYPE            (be_file_menu_item_get_type())
#define BE_FILE_MENU_ITEM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), BE_FILE_MENU_ITEM_TYPE, BeFileMenuItem))
#define BE_FILE_MENU_ITEM_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), BE_FILE_MENU_ITEM_TYPE, BeFileMenuItemClass))
#define BE_IS_FILE_MENU_ITEM(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BE_FILE_MENU_ITEM_TYPE))
#define BE_IS_FILE_MENU_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BE_FILE_MENU_ITEM_TYPE))
#define BE_FILE_MENU_ITEM_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), BE_FILE_MENU_ITEM_TYPE, BeFileMenuItemClass))

typedef struct _BeFileMenuItem BeFileMenuItem;
typedef struct _BeFileMenuItemClass BeFileMenuItemClass;

typedef enum {
	BE_FILE_MENU_ITEM_ACTION_NONE = 0,
	BE_FILE_MENU_ITEM_ACTION_OPEN, /* use action_data if non-NULL, otherwise uri */
	BE_FILE_MENU_ITEM_ACTION_SPAWN /* use action_data */
} BeFileMenuItemActionType;

struct _BeFileMenuItem {
	GtkImageMenuItem parent;

	/* public */
	gchar        *uri;
	BeIconLoader *icon_loader;
	GtkIconSize   icon_size;

	/* private */
	gchar                    *name;
	gchar                    *icon;
	BeFileMenuItemActionType  action_type;
	gpointer                  action_data;
};

struct _BeFileMenuItemClass {
	GtkImageMenuItemClass parent;

	/* class members */
	void (*set_text)(BeFileMenuItem *);
	void (*set_image)(BeFileMenuItem *);
};

GType be_file_menu_item_get_type(void);

GtkWidget *be_file_menu_item_new(const gchar *uri, BeIconLoader *icon_loader, GtkIconSize icon_size);

G_CONST_RETURN gchar *be_file_menu_item_get_name(const BeFileMenuItem *file_menu_item);

void be_file_menu_item_set_uri(BeFileMenuItem *file_menu_item, const gchar *uri);
G_CONST_RETURN gchar *be_file_menu_item_get_uri(const BeFileMenuItem *file_menu_item);

void be_file_menu_item_set_icon_loader(BeFileMenuItem *file_menu_item, BeIconLoader *icon_loader);
BeIconLoader *be_file_menu_item_get_icon_loader(const BeFileMenuItem *file_menu_item);

void be_file_menu_item_set_icon_size(BeFileMenuItem *file_menu_item, GtkIconSize icon_size);
GtkIconSize be_file_menu_item_get_icon_size(const BeFileMenuItem *file_menu_item);

#endif
