#include "config.h"

#include "main-menu-button.h"

#include "main-menu.h"

#include <libgnome/gnome-i18n.h>

enum {
	BE_MAIN_MENU_BUTTON_PROPERTY_ICON_LOADER = 1
};

static void be_main_menu_button_menu_position(GtkMenu *menu, int *x, int *y, gboolean *push_in, gpointer user_data) {
	GtkWidget *widget = GTK_WIDGET(user_data);
	GtkRequisition requisition;
	gint menu_xpos;
	gint menu_ypos;

	gtk_widget_size_request(GTK_WIDGET(menu), &requisition);

	gdk_window_get_origin(widget->window, &menu_xpos, &menu_ypos);

	menu_xpos += widget->allocation.x;
	menu_ypos += widget->allocation.y;

	if(menu_ypos > gdk_screen_get_height(gtk_widget_get_screen(widget)) / 2)
		menu_ypos -= requisition.height;
	else
		menu_ypos += widget->allocation.height;

	*x = menu_xpos;
	*y = menu_ypos;
	*push_in = TRUE;
}

static void be_main_menu_button_select(GtkItem *item) {
	BeMainMenuButton      *main_menu_button = BE_MAIN_MENU_BUTTON(item);
	BeMainMenuButtonClass *klass            = BE_MAIN_MENU_BUTTON_CLASS(g_type_class_peek(BE_MAIN_MENU_BUTTON_TYPE));
	GtkItemClass          *parent_class     = GTK_ITEM_CLASS(g_type_class_peek_parent(klass));

	/* Invoke parent select */
	parent_class->select(item);

	/* Do stuff */
	gtk_menu_popup(GTK_MENU(main_menu_button->main_menu), NULL, NULL, &be_main_menu_button_menu_position, main_menu_button, 0, gtk_get_current_event_time());
}
static void be_main_menu_button_deselect(GtkItem *item) {
	BeMainMenuButton      *main_menu_button = BE_MAIN_MENU_BUTTON(item);
	BeMainMenuButtonClass *klass            = BE_MAIN_MENU_BUTTON_CLASS(g_type_class_peek(BE_MAIN_MENU_BUTTON_TYPE));
	GtkItemClass          *parent_class     = GTK_ITEM_CLASS(g_type_class_peek_parent(klass));

	/* Invoke parent deselect */
	parent_class->deselect(item);

	/* Do stuff */
	gtk_menu_popdown(GTK_MENU(main_menu_button->main_menu));
}

static void be_main_menu_button_menu_selection_done(GtkMenuShell *menu_shell, BeMainMenuButton *main_menu_button) {
	gtk_item_deselect(GTK_ITEM(main_menu_button));
}

static void be_main_menu_button_instance_init(GTypeInstance *instance, gpointer g_class) {
	BeMainMenuButton *self = (BeMainMenuButton *) instance;

	/* Do stuff */
	self->icon_loader = NULL;
	self->main_menu   = be_main_menu_new(NULL);

	g_object_ref(G_OBJECT(self->main_menu));
	gtk_object_sink(GTK_OBJECT(self->main_menu));

	g_signal_connect(self->main_menu, "selection_done", G_CALLBACK(be_main_menu_button_menu_selection_done), self);
}

static void be_main_menu_button_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec) {
	BeMainMenuButton *self = BE_MAIN_MENU_BUTTON(object);

	switch(property_id) {
		case BE_MAIN_MENU_BUTTON_PROPERTY_ICON_LOADER:
			be_main_menu_button_set_icon_loader(self, BE_ICON_LOADER(g_value_get_object(value)));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
			break;
	}
}
static void be_main_menu_button_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec) {
	BeMainMenuButton *self = BE_MAIN_MENU_BUTTON(object);

	switch(property_id) {
		case BE_MAIN_MENU_BUTTON_PROPERTY_ICON_LOADER:
			g_value_set_object(value, self->icon_loader);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
			break;
	}
}

static GObject *be_main_menu_button_constructor(GType type, guint n_construct_properties, GObjectConstructParam *construct_properties) {
	GObject *obj;

	{ /* Invoke parent constructor. */
		BeMainMenuButtonClass *klass        = BE_MAIN_MENU_BUTTON_CLASS(g_type_class_peek(BE_MAIN_MENU_BUTTON_TYPE));
		GObjectClass          *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

		obj = parent_class->constructor(type, n_construct_properties, construct_properties);
	}

	/* Do stuff */

	return obj;
}

static void be_main_menu_button_dispose(GObject *object) {
	BeMainMenuButton      *self         = BE_MAIN_MENU_BUTTON(object);
	BeMainMenuButtonClass *klass        = BE_MAIN_MENU_BUTTON_CLASS(g_type_class_peek(BE_MAIN_MENU_BUTTON_TYPE));
	GObjectClass          *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */
	if(self->icon_loader) {
		g_object_unref(G_OBJECT(self->icon_loader));
		self->icon_loader = NULL;
	}

	/* Invoke parent dispose */
	parent_class->dispose(object);
}

static void be_main_menu_button_finalize(GObject *object) {
	BeMainMenuButton      *self         = BE_MAIN_MENU_BUTTON(object);
	BeMainMenuButtonClass *klass        = BE_MAIN_MENU_BUTTON_CLASS(g_type_class_peek(BE_MAIN_MENU_BUTTON_TYPE));
	GObjectClass          *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */
	g_object_unref(self->main_menu);

	/* Invoke parent finalize */
	parent_class->finalize(object);
}

static void be_main_menu_button_class_init(gpointer g_class, gpointer g_class_data) {
	GObjectClass            *gobject_class   = G_OBJECT_CLASS(g_class);
	GtkItemClass            *item_class      = GTK_ITEM_CLASS(g_class);
/*	BeMainMenuButtonClass   *klass           = BE_MAIN_MENU_BUTTON_CLASS(g_class); */
	GParamSpec              *pspec;

	gobject_class->constructor  = &be_main_menu_button_constructor;
	gobject_class->dispose      = &be_main_menu_button_dispose;
	gobject_class->finalize     = &be_main_menu_button_finalize;
	gobject_class->set_property = &be_main_menu_button_set_property;
	gobject_class->get_property = &be_main_menu_button_get_property;

	pspec = g_param_spec_object("icon-loader",
		"Icon loader",
		"Icon loader to use for requesting images",
		G_TYPE_OBJECT,
		G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class,
		BE_MAIN_MENU_BUTTON_PROPERTY_ICON_LOADER,
		pspec);

	item_class->select   = &be_main_menu_button_select;
	item_class->deselect = &be_main_menu_button_deselect;
}

GType be_main_menu_button_get_type(void) {
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo info = {
			sizeof(BeMainMenuButtonClass),      /* class_size */
			NULL,                               /* base_init */
			NULL,                               /* base_finalize */
			&be_main_menu_button_class_init,    /* class_init */
			NULL,                               /* class_destroy */
			NULL,                               /* class_data */
			sizeof(BeMainMenuButton),           /* instance_size */
			0,                                  /* n_preallocs */
			&be_main_menu_button_instance_init, /* instance_init */
			NULL                                /* value_table */
		};

		type = g_type_register_static(BE_MENU_BUTTON_TYPE, "BeMainMenuButtonType", &info, 0);
	}

	return type;
}

GtkWidget *be_main_menu_button_new(BeIconLoader *icon_loader) {
	GtkWidget *main_menu_button = gtk_widget_new(BE_MAIN_MENU_BUTTON_TYPE,
		"icon-loader", icon_loader,
		NULL);

	return main_menu_button;
}

void be_main_menu_button_set_icon_loader(BeMainMenuButton *main_menu_button, BeIconLoader *icon_loader) {
	if(main_menu_button->icon_loader)
		g_object_unref(G_OBJECT(main_menu_button->icon_loader));

	main_menu_button->icon_loader = icon_loader;

	if(main_menu_button->icon_loader)
		g_object_ref(G_OBJECT(main_menu_button->icon_loader));

	be_main_menu_set_icon_loader(BE_MAIN_MENU(main_menu_button->main_menu), main_menu_button->icon_loader);

	g_object_notify(G_OBJECT(main_menu_button), "icon-loader");
}
