#include "config.h"

#include "main-menu.h"

#include <libgnome/gnome-i18n.h>
#include <libgnomeui/libgnomeui.h>

#include "custom-file-menu-item.h"
#include "file-menu-item.h"
#include "panel-stock-icons.h"

enum {
	BE_MAIN_MENU_PROPERTY_ICON_LOADER = 1
};

#define BE_MAIN_MENU_ICON_SIZE (panel_menu_icon_get_size())

void be_main_menu_search_activate(GtkWidget *widget, gpointer user_data) {
	GdkScreen *screen  = gtk_widget_get_screen(widget);
	gchar     *argv[2] = { "gnome-search-tool", NULL };

	if(!gdk_spawn_on_screen(screen, NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL))
		g_warning("Could not spawn gnome-search-tool\n");
}
void be_main_menu_screenshot_activate(GtkWidget *widget, gpointer user_data) {
	GdkScreen *screen  = gtk_widget_get_screen(widget);
	gchar     *argv[2] = { "gnome-panel-screenshot", NULL };

	if(!gdk_spawn_on_screen(screen, NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL))
		g_warning("Could not spawn gnome-panel-screenshot\n");
}
void be_main_menu_lock_activate(GtkWidget *widget, gpointer user_data) {
	GdkScreen *screen  = gtk_widget_get_screen(widget);
	gchar     *argv[3] = { "xscreensaver-command", "-lock", NULL };

	if(!gdk_spawn_on_screen(screen, NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL))
		g_warning("Could not spawn xscreensaver-command\n");
}
void be_main_menu_logout_activate(GtkWidget *widget, gpointer user_data) {
	GnomeClient *client = gnome_master_client();

	if(client)
		gnome_client_request_save(client, GNOME_SAVE_GLOBAL, TRUE, GNOME_INTERACT_ANY, FALSE, TRUE);
	else
		g_warning("Could not get GNOME master client\n");
}

static void be_main_menu_instance_init(GTypeInstance *instance, gpointer g_class) {
	BeMainMenu *self = (BeMainMenu *) instance;
	GtkWidget  *widget;
	gchar      *tmp;

	self->icon_loader = NULL;

	self->tooltips = gtk_tooltips_new();
	g_object_ref(G_OBJECT(self->tooltips));
	gtk_object_sink(GTK_OBJECT(self->tooltips));

	/* Applications */
	self->applications = be_file_menu_item_new("applications:/", self->icon_loader, BE_MAIN_MENU_ICON_SIZE);
	be_readyable_ready(BE_READYABLE(self->applications));
	gtk_widget_show(self->applications);
	gtk_menu_shell_append(GTK_MENU_SHELL(self), self->applications);

	/* Seperator */
	widget = gtk_separator_menu_item_new();
	gtk_widget_show(widget);
	gtk_menu_shell_append(GTK_MENU_SHELL(self), widget);

	/* Home */
	self->home = be_custom_file_menu_item_new(g_get_home_dir(), self->icon_loader, BE_MAIN_MENU_ICON_SIZE);
	be_custom_file_menu_item_set_text(BE_CUSTOM_FILE_MENU_ITEM(self->home), _("Home"));
	be_custom_file_menu_item_set_image_from_stock(BE_CUSTOM_FILE_MENU_ITEM(self->home), GTK_STOCK_HOME);
	gtk_widget_show(self->home);
	gtk_menu_shell_append(GTK_MENU_SHELL(self), self->home);

	/* Search */
	if((tmp = g_find_program_in_path("gnome-search-tool")) != NULL) {
		g_free(tmp);

		widget = gtk_image_menu_item_new_with_label(_("Search for Files..."));
		gtk_tooltips_set_tip(self->tooltips, widget, _("Find files, folders, and documents on your computer"), NULL);
		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(widget), gtk_image_new_from_stock(PANEL_STOCK_SEARCHTOOL, BE_MAIN_MENU_ICON_SIZE));

		g_signal_connect(widget, "activate", G_CALLBACK(be_main_menu_search_activate), NULL);

		gtk_widget_show(widget);
		gtk_menu_shell_append(GTK_MENU_SHELL(self), widget);
	}

	/* Screenshot */
	if((tmp = g_find_program_in_path("gnome-panel-screenshot")) != NULL) {
		g_free(tmp);

		/* Seperator */
		widget = gtk_separator_menu_item_new();
		gtk_widget_show(widget);
		gtk_menu_shell_append(GTK_MENU_SHELL(self), widget);

		widget = gtk_image_menu_item_new_with_label(_("Take Screenshot..."));
		gtk_tooltips_set_tip(self->tooltips, widget, _("Take a screenshot of your desktop"), NULL);
		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(widget), gtk_image_new_from_stock(PANEL_STOCK_SCREENSHOT, BE_MAIN_MENU_ICON_SIZE));

		g_signal_connect(widget, "activate", G_CALLBACK(be_main_menu_screenshot_activate), NULL);

		gtk_widget_show(widget);
		gtk_menu_shell_append(GTK_MENU_SHELL(self), widget);
	}

	/* Seperator */
	widget = gtk_separator_menu_item_new();
	gtk_widget_show(widget);
	gtk_menu_shell_append(GTK_MENU_SHELL(self), widget);

	/* Lock */
	if((tmp = g_find_program_in_path("xscreensaver-command")) != NULL) {
		g_free(tmp);

		widget = gtk_image_menu_item_new_with_label(_("Lock Screen"));
		gtk_tooltips_set_tip(self->tooltips, widget, _("Protect your computer from unauthorized use"), NULL);
		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(widget), gtk_image_new_from_stock(PANEL_STOCK_LOCKSCREEN, BE_MAIN_MENU_ICON_SIZE));

		g_signal_connect(widget, "activate", G_CALLBACK(be_main_menu_lock_activate), NULL);

		gtk_widget_show(widget);
		gtk_menu_shell_append(GTK_MENU_SHELL(self), widget);
	}

	/* Log out */
	widget = gtk_image_menu_item_new_with_label(_("Log out"));
	gtk_tooltips_set_tip(self->tooltips, widget, _("Log out of GNOME"), NULL);
	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(widget), gtk_image_new_from_stock(PANEL_STOCK_LOGOUT, BE_MAIN_MENU_ICON_SIZE));

	g_signal_connect(widget, "activate", G_CALLBACK(be_main_menu_logout_activate), NULL);

	gtk_widget_show(widget);
	gtk_menu_shell_append(GTK_MENU_SHELL(self), widget);
}

static void be_main_menu_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec) {
	BeMainMenu *self = BE_MAIN_MENU(object);

	switch(property_id) {
		case BE_MAIN_MENU_PROPERTY_ICON_LOADER:
			be_main_menu_set_icon_loader(self, BE_ICON_LOADER(g_value_get_object(value)));
			break;
		default:
			g_assert_not_reached();
			break;
	}
}
static void be_main_menu_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec) {
	BeMainMenu *self = BE_MAIN_MENU(object);

	switch(property_id) {
		case BE_MAIN_MENU_PROPERTY_ICON_LOADER:
			g_value_set_object(value, self->icon_loader);
			break;
		default:
			g_assert_not_reached();
			break;
	}
}

static GObject *be_main_menu_constructor(GType type, guint n_construct_properties, GObjectConstructParam *construct_properties) {
	GObject *obj;

	{ /* Invoke parent constructor. */
		BeMainMenuClass *klass        = BE_MAIN_MENU_CLASS(g_type_class_peek(BE_MAIN_MENU_TYPE));
		GObjectClass    *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

		obj = parent_class->constructor(type, n_construct_properties, construct_properties);
	}

	/* Do stuff */

	return obj;
}

static void be_main_menu_dispose(GObject *object) {
	BeMainMenu      *self         = BE_MAIN_MENU(object);
	BeMainMenuClass *klass        = BE_MAIN_MENU_CLASS(g_type_class_peek(BE_MAIN_MENU_TYPE));
	GObjectClass    *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */
	if(self->icon_loader) {
		g_object_unref(G_OBJECT(self->icon_loader));
		self->icon_loader = NULL;
	}

	/* Invoke parent dispose */
	parent_class->dispose(object);
}

static void be_main_menu_finalize(GObject *object) {
	BeMainMenu      *self         = BE_MAIN_MENU(object);
	BeMainMenuClass *klass        = BE_MAIN_MENU_CLASS(g_type_class_peek(BE_MAIN_MENU_TYPE));
	GObjectClass    *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */
	g_object_unref(G_OBJECT(self->tooltips));

	/* Invoke parent finalize */
	parent_class->finalize(object);
}

static void be_main_menu_class_init(gpointer g_class, gpointer g_class_data) {
	GObjectClass    *gobject_class = G_OBJECT_CLASS(g_class);
/*	BeMainMenuClass *klass         = BE_MAIN_MENU_CLASS(g_class); */
	GParamSpec      *pspec;

	gobject_class->constructor  = &be_main_menu_constructor;
	gobject_class->dispose      = &be_main_menu_dispose;
	gobject_class->finalize     = &be_main_menu_finalize;
	gobject_class->set_property = &be_main_menu_set_property;
	gobject_class->get_property = &be_main_menu_get_property;

	pspec = g_param_spec_object("icon-loader",
		"Icon loader",
		"Icon loader to use for requesting images",
		G_TYPE_OBJECT,
		G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class,
		BE_MAIN_MENU_PROPERTY_ICON_LOADER,
		pspec);
}

GType be_main_menu_get_type(void) {
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo info = {
			sizeof(BeMainMenuClass),     /* class_size */
			NULL,                        /* base_init */
			NULL,                        /* base_finalize */
			&be_main_menu_class_init,    /* class_init */
			NULL,                        /* class_destroy */
			NULL,                        /* class_data */
			sizeof(BeMainMenu),          /* instance_size */
			0,                           /* n_preallocs */
			&be_main_menu_instance_init, /* instance_init */
			NULL                         /* value_table */
		};

		type = g_type_register_static(GTK_TYPE_MENU, "BeMainMenuType", &info, 0);
	}

	return type;
}

GtkWidget *be_main_menu_new(BeIconLoader *icon_loader) {
	GtkWidget *main_menu = gtk_widget_new(BE_MAIN_MENU_TYPE,
		"icon-loader", icon_loader,
		NULL);

	return main_menu;
}

void be_main_menu_set_icon_loader(BeMainMenu *main_menu, BeIconLoader *icon_loader) {
	if(main_menu->icon_loader)
		g_object_unref(G_OBJECT(main_menu->icon_loader));

	main_menu->icon_loader = icon_loader;

	if(main_menu->icon_loader)
		g_object_ref(G_OBJECT(main_menu->icon_loader));

	/* Set children */
	if(main_menu->applications)
		be_file_menu_item_set_icon_loader(BE_FILE_MENU_ITEM(main_menu->applications), icon_loader);
	if(main_menu->home)
		be_file_menu_item_set_icon_loader(BE_FILE_MENU_ITEM(main_menu->home), icon_loader);

	g_object_notify(G_OBJECT(main_menu), "icon-loader");
}
