#include "config.h"

#include "window-menu-button.h"

#include "window-menu.h"

/* Thanks gnome-panel, once again. */

#include <libgnome/gnome-i18n.h>

static void be_window_menu_button_menu_position(GtkMenu *menu, int *x, int *y, gboolean *push_in, gpointer user_data) {
	GtkWidget *widget = GTK_WIDGET(user_data);
	GtkRequisition requisition;
	gint menu_xpos;
	gint menu_ypos;

	gtk_widget_size_request(GTK_WIDGET(menu), &requisition);

	gdk_window_get_origin(widget->window, &menu_xpos, &menu_ypos);

	menu_xpos += widget->allocation.x;
	menu_ypos += widget->allocation.y;

	if(menu_ypos > gdk_screen_get_height(gtk_widget_get_screen(widget)) / 2)
		menu_ypos -= requisition.height;
	else
		menu_ypos += widget->allocation.height;

	*x = menu_xpos;
	*y = menu_ypos;
	*push_in = TRUE;
}

static void be_window_menu_button_select(GtkItem *item) {
	BeWindowMenuButton      *window_menu_button = BE_WINDOW_MENU_BUTTON(item);
	BeWindowMenuButtonClass *klass              = BE_WINDOW_MENU_BUTTON_CLASS(g_type_class_peek(BE_WINDOW_MENU_BUTTON_TYPE));
	GtkItemClass            *parent_class       = GTK_ITEM_CLASS(g_type_class_peek_parent(klass));

	/* Invoke parent popup */
	parent_class->select(item);

	/* Do stuff */
	gtk_menu_popup(GTK_MENU(window_menu_button->window_menu), NULL, NULL, &be_window_menu_button_menu_position, window_menu_button, 0, gtk_get_current_event_time());
}
static void be_window_menu_button_deselect(GtkItem *item) {
	BeWindowMenuButton      *window_menu_button = BE_WINDOW_MENU_BUTTON(item);
	BeWindowMenuButtonClass *klass              = BE_WINDOW_MENU_BUTTON_CLASS(g_type_class_peek(BE_WINDOW_MENU_BUTTON_TYPE));
	GtkItemClass            *parent_class       = GTK_ITEM_CLASS(g_type_class_peek_parent(klass));

	/* Invoke parent popdown */
	parent_class->deselect(item);

	/* Do stuff */
	gtk_menu_popdown(GTK_MENU(window_menu_button->window_menu));
}

static void be_window_menu_button_menu_selection_done(GtkMenuShell *menu_shell, BeWindowMenuButton *window_menu_button) {
	gtk_item_deselect(GTK_ITEM(window_menu_button));
}

static void be_window_menu_button_instance_init(GTypeInstance *instance, gpointer g_class) {
	BeWindowMenuButton *self       = (BeWindowMenuButton *) instance;
	AtkObject          *atk_object = gtk_widget_get_accessible(GTK_WIDGET(self));

	/* Do stuff */
	self->window_menu = be_window_menu_new();

	g_object_ref(G_OBJECT(self->window_menu));
	gtk_object_sink(GTK_OBJECT(self->window_menu));

	atk_object_set_name(atk_object, _("Window Selector"));
	atk_object_set_description(atk_object, _("Tool to switch between windows"));

	g_signal_connect(self->window_menu, "selection_done", G_CALLBACK(be_window_menu_button_menu_selection_done), self);
}

static GObject *be_window_menu_button_constructor(GType type, guint n_construct_properties, GObjectConstructParam *construct_properties) {
	GObject *obj;

	{ /* Invoke parent constructor. */
		BeWindowMenuButtonClass *klass        = BE_WINDOW_MENU_BUTTON_CLASS(g_type_class_peek(BE_WINDOW_MENU_BUTTON_TYPE));
		GObjectClass            *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

		obj = parent_class->constructor(type, n_construct_properties, construct_properties);
	}

	/* Do stuff */

	return obj;
}

#if 0
static void be_window_menu_button_dispose(GObject *object) {
	BeWindowMenuButtonClass *klass        = BE_WINDOW_MENU_BUTTON_CLASS(g_type_class_peek(BE_WINDOW_MENU_BUTTON_TYPE));
	GObjectClass            *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */

	/* Invoke parent dispose */
	parent_class->dispose(object);
}
#endif

static void be_window_menu_button_finalize(GObject *object) {
	BeWindowMenuButton      *self         = BE_WINDOW_MENU_BUTTON(object);
	BeWindowMenuButtonClass *klass        = BE_WINDOW_MENU_BUTTON_CLASS(g_type_class_peek(BE_WINDOW_MENU_BUTTON_TYPE));
	GObjectClass            *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */
	g_object_unref(self->window_menu);

	/* Invoke parent finalize */
	parent_class->finalize(object);
}

static void be_window_menu_button_class_init(gpointer g_class, gpointer g_class_data) {
	GObjectClass            *gobject_class     = G_OBJECT_CLASS(g_class);
	GtkItemClass            *item_class        = GTK_ITEM_CLASS(g_class);
/*	BeWindowMenuButtonClass *klass             = BE_WINDOW_MENU_BUTTON_CLASS(g_class); */

	gobject_class->constructor = &be_window_menu_button_constructor;
/*	gobject_class->dispose     = &be_window_menu_button_dispose; */
	gobject_class->finalize    = &be_window_menu_button_finalize;

	item_class->select   = &be_window_menu_button_select;
	item_class->deselect = &be_window_menu_button_deselect;
}

GType be_window_menu_button_get_type(void) {
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo info = {
			sizeof(BeWindowMenuButtonClass),      /* class_size */
			NULL,                                 /* base_init */
			NULL,                                 /* base_finalize */
			&be_window_menu_button_class_init,    /* class_init */
			NULL,                                 /* class_destroy */
			NULL,                                 /* class_data */
			sizeof(BeWindowMenuButton),           /* instance_size */
			0,                                    /* n_preallocs */
			&be_window_menu_button_instance_init, /* instance_init */
			NULL                                  /* value_table */
		};

		type = g_type_register_static(BE_MENU_BUTTON_TYPE, "BeWindowMenuButtonType", &info, 0);
	}

	return type;
}

GtkWidget *be_window_menu_button_new(void) {
	GtkWidget *window_menu_button = gtk_widget_new(BE_WINDOW_MENU_BUTTON_TYPE,
		NULL);

	return window_menu_button;
}
