#include "config.h"

#include "window-menu.h"

/* Thanks to gnome-panel for much the source code that I based this upon. */

#define WNCK_I_KNOW_THIS_IS_UNSTABLE
#include <eel/eel-ellipsizing-label.h>
#include <libgnome/gnome-i18n.h>
#include <libgnomeui/libgnomeui.h>
#include <libwnck/libwnck.h>

#define BE_MAX_TEXT_WIDTH 50

static void be_connect_while_alive(gpointer object, const gchar *signal, GCallback func, gpointer func_data, GObject *alive_object) {
	GClosure *closure;

	closure = g_cclosure_new(func, func_data, NULL);
	g_object_watch_closure(alive_object, closure);
	g_signal_connect_closure_by_id(object, g_signal_lookup(signal, G_OBJECT_TYPE(object)), 0, closure, FALSE);
}

static gint be_get_text_width(GtkWidget *widget, const gchar *text) {
	PangoContext *context;
	PangoFontMetrics *metrics;
	PangoLayout *layout;
	PangoRectangle natural;
	gint char_width;
	gint max_width;
	gint screen_width;
	gint width;

	gtk_widget_ensure_style(widget);

	context = gtk_widget_get_pango_context(widget);
	metrics = pango_context_get_metrics(context, widget->style->font_desc, pango_context_get_language(context));

	char_width = pango_font_metrics_get_approximate_char_width(metrics);

	pango_font_metrics_unref(metrics);

	max_width = PANGO_PIXELS(BE_MAX_TEXT_WIDTH * char_width);

	layout = gtk_widget_create_pango_layout(widget, text);

	pango_layout_get_pixel_extents(layout, NULL, &natural);

	g_object_unref(G_OBJECT(layout));

	screen_width = gdk_screen_get_width(gtk_widget_get_screen(widget));

	width = MIN(natural.width, max_width);
	width = MIN(width, 3 * (screen_width / 4));

	return width;
}

static WnckScreen *be_window_menu_item_get_screen(BeWindowMenu *window_menu) {
	return wnck_screen_get(gdk_screen_get_number(gtk_widget_get_screen(GTK_WIDGET(window_menu))));
}

static GdkPixbuf *be_window_menu_get_icon(BeWindowMenu *window_menu, WnckWindow *window) {
	GdkPixbuf *pixbuf = NULL;
	gint       width, height;
	gint       icon_size;

	gtk_icon_size_lookup(GTK_ICON_SIZE_MENU, NULL, &icon_size);

	if(window)
		pixbuf = wnck_window_get_icon(window);

	if(!pixbuf)
		pixbuf = window_menu->no_icon;

	g_object_ref(pixbuf);

	width  = gdk_pixbuf_get_width(pixbuf);
	height = gdk_pixbuf_get_height(pixbuf);

	if(width > icon_size || height > icon_size) {
		double scale = ((double) icon_size) / MAX(width, height);
		GdkPixbuf *new_pixbuf;

		new_pixbuf = gdk_pixbuf_scale_simple(pixbuf, width * scale, height * scale, GDK_INTERP_BILINEAR);
		g_object_unref(pixbuf);
		pixbuf = new_pixbuf;
	}

	return pixbuf;
}

static GtkWidget *be_window_menu_item_label_new(BeWindowMenu *window_menu, GdkPixbuf *pixbuf, const gchar *text) {
	GtkWidget *item;
	GtkWidget *ellipsizing_label;

	item = gtk_image_menu_item_new();

	ellipsizing_label = eel_ellipsizing_label_new(text);
	gtk_misc_set_alignment(GTK_MISC(ellipsizing_label), 0.0, 0.5);
	gtk_widget_set_size_request(ellipsizing_label, be_get_text_width(GTK_WIDGET(window_menu), text), -1);
	gtk_widget_show(ellipsizing_label);

	gtk_container_add(GTK_CONTAINER(item), ellipsizing_label);

	if(pixbuf) {
		GtkWidget *image = gtk_image_new_from_pixbuf(pixbuf);

		gtk_widget_show(image);

		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
	}

	return item;
}

static GtkWidget *be_window_menu_no_windows_new(BeWindowMenu *window_menu) {
	GtkWidget *no_windows = be_window_menu_item_label_new(window_menu, NULL, _("No Windows Open"));

	g_object_ref(no_windows);
	gtk_object_sink(GTK_OBJECT(no_windows));

	return no_windows;
}

static GtkWidget *be_window_menu_separator_new(void) {
	GtkWidget *separator = gtk_separator_menu_item_new();

	g_object_ref(separator);
	gtk_object_sink(GTK_OBJECT(separator));

	return separator;
}

static GdkPixbuf *be_window_menu_no_icon_new(void) {
	GdkPixbuf *pixbuf;
	gchar *default_icon_path;

	/* FIXME */
	default_icon_path = g_strdup("/usr/share/be-applet/default_icon.png");
	/* gnome_program_locate_file(NULL, GNOME_FILE_DOMAIN_APP_DATADIR, "default_icon.png", TRUE, NULL); */

	pixbuf = gdk_pixbuf_new_from_file(default_icon_path, NULL);

	g_free(default_icon_path);

	return pixbuf;
}

static void be_window_menu_window_icon_changed(WnckWindow *window, BeWindowMenu *window_menu) {
	GtkWidget *item = g_hash_table_lookup(window_menu->windows, window);

	if(item) {
		GdkPixbuf *pixbuf = be_window_menu_get_icon(window_menu, window);
		GtkWidget *image  = gtk_image_new_from_pixbuf(pixbuf);

		gtk_widget_show(image);

		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);

		g_object_unref(pixbuf);
	}
}

static void be_window_menu_window_activate(GtkMenuItem *item, WnckWindow *window) {
	WnckWorkspace *workspace = wnck_window_get_workspace(window);

	wnck_workspace_activate(workspace);

	if(wnck_window_is_minimized(window))
		wnck_window_unminimize(window);

	wnck_window_activate(window);
}

static GtkWidget *be_window_menu_window_new(BeWindowMenu *window_menu, WnckWindow *window) {
	GtkWidget *item = NULL;

	if(!wnck_window_is_skip_tasklist(window)) {
		GdkPixbuf   *pixbuf;
		const gchar *name;

		pixbuf = be_window_menu_get_icon(window_menu, window);

		name = wnck_window_get_name(window);
		if(!name)
			name = _("Unknown Window");

		if(wnck_window_is_minimized(window)) {
			gchar *min_name = g_strdup_printf("[%s]", name);

			item = be_window_menu_item_label_new(window_menu, pixbuf, min_name);

			g_free(min_name);
		} else {
			item = be_window_menu_item_label_new(window_menu, pixbuf, name);
		}

		g_object_unref(pixbuf);

		be_connect_while_alive(window, "icon_changed", G_CALLBACK(be_window_menu_window_icon_changed), window_menu, G_OBJECT(window_menu));
		g_signal_connect(item, "activate", G_CALLBACK(be_window_menu_window_activate), window);
	}

	return item;
}

static void be_window_menu_append(BeWindowMenu *window_menu, WnckWindow *window) {
	GtkWidget *item = be_window_menu_window_new(window_menu, window);

	if(item) {
		gtk_widget_show(item);

		/* Check if it's on this workspace or another */
		if(wnck_screen_get_active_workspace(be_window_menu_item_get_screen(window_menu)) == wnck_window_get_workspace(window))
			gtk_menu_shell_prepend(GTK_MENU_SHELL(window_menu), item);
		else
			gtk_menu_shell_append(GTK_MENU_SHELL(window_menu), item);

		g_hash_table_insert(window_menu->windows, window, item);

		if(GTK_WIDGET_VISIBLE(window_menu->no_windows)) {
			gtk_widget_hide(window_menu->no_windows);
			gtk_widget_show(window_menu->separator);
		}
		if(GTK_WIDGET_VISIBLE(window_menu))
			gtk_menu_reposition(GTK_MENU(window_menu));
	}
}

static void be_window_menu_remove(BeWindowMenu *window_menu, WnckWindow *window) {
	GtkWidget *item = g_hash_table_lookup(window_menu->windows, window);

	if(item) {
		gtk_container_remove(GTK_CONTAINER(window_menu), item);

		g_hash_table_remove(window_menu->windows, window);

		if(g_hash_table_size(window_menu->windows) == 0) {
			gtk_widget_hide(window_menu->separator);
			gtk_widget_show(window_menu->no_windows);
		}
		if(GTK_WIDGET_VISIBLE(window_menu))
			gtk_menu_reposition(GTK_MENU(window_menu));
	}
}

static gboolean be_window_menu_clear_hrfunc(gpointer key, gpointer value, gpointer user_data) {
	BeWindowMenu *window_menu = user_data;

	gtk_container_remove(GTK_CONTAINER(window_menu), GTK_WIDGET(value));

	return TRUE;
}

static void be_window_menu_clear(BeWindowMenu *window_menu) {
	g_hash_table_foreach_remove(window_menu->windows, &be_window_menu_clear_hrfunc, window_menu);
}

static void be_window_menu_fill(BeWindowMenu *window_menu) {
	WnckScreen *screen = be_window_menu_item_get_screen(window_menu);
	GList      *cur, *windows;

	windows = wnck_screen_get_windows(screen);

	if(windows) {
		for(cur = windows; cur; cur = g_list_next(cur))
			be_window_menu_append(window_menu, cur->data);
		
		gtk_widget_hide(window_menu->no_windows);
		gtk_widget_show(window_menu->separator);
	} else {
		gtk_widget_hide(window_menu->separator);
		gtk_widget_show(window_menu->no_windows);
	}
}

static void be_window_menu_refresh(BeWindowMenu *window_menu) {
	be_window_menu_clear(window_menu);
	be_window_menu_fill(window_menu);
}

static void be_window_menu_setup(BeWindowMenu *window_menu) {
	/* Workspace separator and no windows label */
	gtk_menu_shell_append(GTK_MENU_SHELL(window_menu), window_menu->no_windows);
	gtk_menu_shell_append(GTK_MENU_SHELL(window_menu), window_menu->separator);

	be_window_menu_fill(window_menu);
}

/* When the menu is shown, refresh it. */
static void be_window_menu_show(GtkWidget *widget, gpointer user_data) {
	BeWindowMenu *window_menu = BE_WINDOW_MENU(widget);

	be_window_menu_refresh(window_menu);
}

/* When the menu is visible, track changes. Otherwise, put them off the the show-time refresh. */
static void be_window_menu_window_opened(WnckScreen *screen, WnckWindow *window, BeWindowMenu *window_menu) {
	if(GTK_WIDGET_VISIBLE(window_menu))
		be_window_menu_append(window_menu, window);
}
static void be_window_menu_window_closed(WnckScreen *screen, WnckWindow *window, BeWindowMenu *window_menu) {
	if(GTK_WIDGET_VISIBLE(window_menu))
		be_window_menu_remove(window_menu, window);
}
static void be_window_menu_active_workspace_changed(WnckWindow *screen, BeWindowMenu *window_menu) {
	if(GTK_WIDGET_VISIBLE(window_menu))
		be_window_menu_refresh(window_menu);
}

static void be_window_menu_instance_init(GTypeInstance *instance, gpointer g_class) {
	BeWindowMenu *self   = (BeWindowMenu *) instance;
	WnckScreen   *screen = be_window_menu_item_get_screen(self);

	/* Do stuff */
	self->windows    = g_hash_table_new(g_direct_hash, g_direct_equal);
	self->no_windows = be_window_menu_no_windows_new(self);
	self->separator  = be_window_menu_separator_new();
	self->no_icon    = be_window_menu_no_icon_new();

	be_window_menu_setup(self);

	g_signal_connect(self, "show", G_CALLBACK(be_window_menu_show), NULL);

	be_connect_while_alive(screen, "window_opened", G_CALLBACK(be_window_menu_window_opened), self, G_OBJECT(self));
	be_connect_while_alive(screen, "window_closed", G_CALLBACK(be_window_menu_window_closed), self, G_OBJECT(self));
	be_connect_while_alive(screen, "active_workspace_changed", G_CALLBACK(be_window_menu_active_workspace_changed), self, G_OBJECT(self));
}

static GObject *be_window_menu_constructor(GType type, guint n_construct_properties, GObjectConstructParam *construct_properties) {
	GObject *obj;

	{ /* Invoke parent constructor. */
		BeWindowMenuClass *klass        = BE_WINDOW_MENU_CLASS(g_type_class_peek(BE_WINDOW_MENU_TYPE));
		GObjectClass      *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

		obj = parent_class->constructor(type, n_construct_properties, construct_properties);
	}

	/* Do stuff */

	return obj;
}

#if 0
static void be_window_menu_dispose(GObject *object) {
	BeWindowMenuClass *klass        = BE_WINDOW_MENU_CLASS(g_type_class_peek(BE_WINDOW_MENU_TYPE));
	GObjectClass      *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */

	/* Invoke parent dispose */
	parent_class->dispose(object);
}
#endif

static void be_window_menu_finalize(GObject *object) {
	BeWindowMenu      *self         = BE_WINDOW_MENU(object);
	BeWindowMenuClass *klass        = BE_WINDOW_MENU_CLASS(g_type_class_peek(BE_WINDOW_MENU_TYPE));
	GObjectClass      *parent_class = G_OBJECT_CLASS(g_type_class_peek_parent(klass));

	/* Do stuff */
	g_hash_table_destroy(self->windows);
	gtk_widget_destroy(self->no_windows);
	gtk_widget_destroy(self->separator);
	g_object_unref(self->no_icon);

	/* Invoke parent finalize */
	parent_class->finalize(object);
}

static void be_window_menu_class_init(gpointer g_class, gpointer g_class_data) {
	GObjectClass      *gobject_class = G_OBJECT_CLASS(g_class);
/*	BeWindowMenuClass *klass         = BE_WINDOW_MENU_CLASS(g_class); */

	gobject_class->constructor = &be_window_menu_constructor;
/*	gobject_class->dispose     = &be_window_menu_dispose; */
	gobject_class->finalize    = &be_window_menu_finalize;
}

GType be_window_menu_get_type(void) {
	static GType type = 0;

	if(type == 0) {
		static const GTypeInfo info = {
			sizeof(BeWindowMenuClass),     /* class_size */
			NULL,                          /* base_init */
			NULL,                          /* base_finalize */
			&be_window_menu_class_init,    /* class_init */
			NULL,                          /* class_destroy */
			NULL,                          /* class_data */
			sizeof(BeWindowMenu),          /* instance_size */
			0,                             /* n_preallocs */
			&be_window_menu_instance_init, /* instance_init */
			NULL                           /* value_table */
		};

		type = g_type_register_static(GTK_TYPE_MENU, "BeWindowMenuType", &info, 0);
	}

	return type;
}

GtkWidget *be_window_menu_new(void) {
	GtkWidget *window_menu = gtk_widget_new(BE_WINDOW_MENU_TYPE, NULL);

	return window_menu;
}
